#!/usr/local/bin/ruby
# cache.rb  ver.0.4  2002.4.7-2003.9.27
# book-info from/to cache
# original; amazon.rb for amazon by not
# cacheƂISBNL[œǂݏׂ̐^
# eXpNXɌpėp

require 'kconv'

class CacheBib
  attr_accessor :info
  attr_reader   :cache_dir
  attr_accessor :expire, :expire_full
  attr_accessor :insystemKcode, :outputKcode, :messageKcode
  attr_accessor :flag_read, :flag_save, :flag_net
  attr_accessor :flag, :flag_lock, :flag_init
  attr_accessor :stderr

  def initialize ()
    @expire      = 12    #detail-reload-time (hour)
    @expire_full = 720   #all-reload-time (hour) (ISBN-search->detail-parse)
    @cache_dir   = "./"  #chache-data-folder
    @insystemKcode = 3   #sjis #1-jis,2-euc,3-sjis
    @outputKcode   = 2   #euc
    @messageKcode  = 3   #sjis
#   @bib = BibInfo.new()
    @flag_read = true
    @flag_save = true
    @flag_net  = true
    @flag_lock = false
    @flag_init = false
    @flag = false
    @stderr = $stderr
  end

#===== ===== read/write chace ===== =====
  def read_cache
    #----- check read-cache-flag -----
    return if @flag #----- already read cache -----
    if File::readable?(@cache_dir + @file)
      unless @expire.nil?
        if File::mtime(@cache_dir + @file).to_i < Time::now.to_i - 3600 * @expire
          @flag = nil #----- cache read and reload -----
        else
          @flag = true #----- cahce read (only) -----
        end
      else
        @flag = true #----- cahce read (only) -----
      end
    end
    
    #----- read-cache -----
    begin
      #----- \n\n-sepalate -----
      texts = []
      text = []
      open(@cache_dir + @file).each{ |line|
        string = (line.chomp.chomp("\r")).kconv(@insystemKcode)
        if string != ""
          text << string
        else
          texts << text
          text = []
        end
      }
      texts << text
      
      #----- book-info -----
      texts.shift.each{ |line|
        record = line.split(/\t/)
        key = record.shift
        data = record.join("\t")
        @info[key] = data
      }
      
      #----- review-info -----
      if texts.size > 0
        reviews = []
        texts.each{ |text|
          reviewinfo = {}
          text.each { |line|
            record = line.split(/\t/)
            key = record.shift
            data = record.join("\t")
            reviewinfo[key] = data
          }
          reviews << reviewinfo
        }
        @info["reviews"] = reviews
      end
    rescue
      message = "CHK; cannot read cahce. FILE; " + @cache_dir.to_s + @file.to_s
      @stderr.puts message.kconv(@messageKcode)
      @flag = nil
    end
  end

  def save_cache
    unless File::directory?(@cache_dir.to_s)
      message = "WAR; no-cache-folder. DIR; " + @cache_dir.to_s
      @stderr.puts message.kconv(@messageKcode)
      return
    end
    
    begin
      open(@cache_dir + @file, 'w'){ |fp|
        #----- book-info -----
        @info.keys.sort.each{ |key|
          data = @info[key]
          if (!data.nil?) && (key != "reviews")
            key  = key.to_s.kconv(@outputKcode)
            data = data.to_s.kconv(@outputKcode)
            fp.puts key + "\t" + data
          end
        }
        #----- review-info -----
        if !@info["reviews"].nil?
          @info["reviews"].each{ |review|
            fp.puts ''
            review.keys.sort.each{ |key|
              data = review[key]
              key  = key.to_s.kconv(@outputKcode)
              data = data.to_s.kconv(@outputKcode)
              fp.puts key + "\t" + data
            }
          }
        end
      }
    rescue
      message = "WAR; cannot write cache. FILE; " + @cache_dir.to_s + @file.to_s
      @stderr.puts message.kconv(@messageKcode)
    end
  end

#===== ===== main ===== =====
  def get_data ( isbn = nil, flag = nil )
    @isbn = isbn_check(isbn) if !isbn.nil?
    @file = "isbn" + @isbn.delete('-').downcase + ".txt"
    @info = {}
    
    #----- check(read)-cache -----
    read_cache if @flag_read
    if @flag || flag #----- read only return -----
      set_info
      return @info
    end
    
    @flag_lock = @info['flag_lock'] unless @info['flag_lock'].nil?
    unless @expire_full.nil?
      if File::readable?(@cache_dir + @file)
        if File::mtime(@cache_dir + @file) < Time::now.to_i - 3600 * @expire_full
          if @flag_lock == false
            @info = {}
          elsif @flag_lock == 'id'
            @info = {'id'=>@info['id']}
          end #=> @flag_lock == 'all'
        end
      end
    end
    if @flag_init
      @info = {} if @flag_read == false
    else
      @info = {'id'=>@info['id']} if @flag_read == false
    end
    set_info
    
    #----- download-bibinfo -----
    get_info   if @flag_net
    
    #----- write-cache -----
    save_cache if @flag_save
    @flag = true
    
    return @info #----- download and write return -----
  end

#===== ===== parts ===== =====
# ToDo: NXϐENX\bh̕H
#       ƓɃIuWFNg\bhł肽̂̕@́H
  def cache_dir=(dir)
    dir << '/' unless dir[-1,1] == '/'
    @cache_dir = dir
  end

  def isbn_check ( isbn )
    isbn2 = isbn.delete('-').upcase
    check = 11
    0.upto(8){ |i| check = check - (isbn2[i,1].to_i * (10-i)) }
    check = check % 11
    digit = check.to_s
    digit = 'X' if check == 10
    digit = '0' if check == 11
    if isbn2.size < 10 #----- check-digit-lost -----
      message = "WAR; ISBN check-digit lost. -> #{digit}"
      @stderr.puts message.kconv(@messageKcode)
      isbn2 << digit
      isbn << '-' unless isbn[-1,1] != '/'
      isbn << digit
    else #----- check-digit-error -----
      check = isbn2[9,1]
      if check != digit
        message = "WAR; ISBN check-digit error. #{check} -> #{digit}"
        @stderr.puts message.kconv(@messageKcode)
        isbn2[9] = digit
        isbn.gsub!(/-.$/,"-#{digit}")
      end
    end
    return isbn
  end

  def merge_info ( list ) # = merge_hash
    info = {}
    list.each { |data_info|
      data_info.each{ |key,data|
        if data.to_s.size > 0
          if info[key].nil?
            info[key] = data
          end
        end
      }
    }
    return info
  end
end

=begin
:cachefile-format:

f[^Ŝ́AsŁyubNzɕB
yubNzif[^Śj
[kl,k]1l,k]2l,c,k]Nl]
ƂȂB
eyubNźAsŁyCzɕB
yCz
k^Ol[TAB]kel[RET]
i[TAB]̓^uA[RET]͉sj
ƂȂB
k^OĺApIȈӖŌŒ肳Ă邪Aɂ͎RłB
igׁ̈j
k^OĺAureviewsv͋֎~i\jĂB
ik]lׁ̈j
kelł́A[TAB][RET]Ӑ}B
ikelɉsꍇA̓^uɒuĕۑĂj
kel[TAB]͕ۑłȂB
ikelɃ^u͊܂߂鎖łȂj

ȉɑ\Iȁk^Ol
code	{IISBNiISBN4-cƂejij
isbn	ISBNiHISBNł͂Ȃԍ̏ꍇLj
id	XԍiEdLłނ떢j
pubcode	oŎГԍ
title	
series	V[YiV[YȂ̂ȂjiB[x̎Lj
author	
translator	
publisher	sioŎЁj
pubdate	s
image	eURL
limage	ȇTCŶ̂URL
price	i
desc	eЉiindex=ڎ܂ގj
delivery	zB
pubsize	TCYiA4œj
e_title	
e_author	ҁij
e_pubdate	síj
reviews	\F̔z񂪊i[B
reviewer	]Җ

֐͏HashŕԂB
info["hoge"]́klhogek^Ol̓eA
info["reviews"][0]["fuga"]́k]1lfugak^Ol̓eB

uinfo.hogevuinfo.reviews[0].fugav̏ɂoɂȂ̂́A
EuŕێĂf[^vƁuۂɏof[^v̊Ԃɍꍇ邾낤imxȂƂ̗RŁj
Eup infov̎gpŏ̈ꗗł낤
ƂRɂB
=end

=begin
:cachefile-reload-pattern:

kf[^lɂ͊e폑񂪊܂܂B
kf[^lɂ͕KkR[hli{IISBNjƁkL[li{IɏXR[hj܂܂B
ł́AkR[hlŌākL[lAkL[lŏ擾B
Ȍł́AkL[lƂɏ擾B

cache̓ǂݏɂĂ͎̏2_łB
flag_read = true:Ǎ
flag_save = true:
Ƃ1_
flag_net = true:lbgڑ

cache̍XVɂĂ͎2̎Ԑݒ肪B
kf[^l͏㏑AkL[lĎ擾ȂFexpire
kf[^l͑SAkL[lĎ擾Fexpire_full
2nilŐݒ肪삵ȂȂB

ɂĂ͎4_łB
ꂩ擾iLbV͓ǂ܂ȂAkR[hl̂ݗ^jFexpire_full
ꂩ擾iLbVǍ񂾏Łj
kL[l擾iLbV̓L[ȊO͓ǂ܂Ȃj
kL[l擾iLbVǍ񂾏ŁjFexpire
lbg擾ȂiLbV̓Ǎ݂̂݁j
flag_init = true(=code), false(=key)Fꂩ擾AkL[l擾
flag_read = true(=read), false(=noread)FLbVǂނAǂ܂Ȃ
flag = true(=nonet), nil(=net)Flbg擾ȂA

kL[lłœ͂ĂꍇikR[hl͌łȂꍇj
̂ŁAexpire_fullłkL[l͎coptionk^OlpӂB
ĺk^OlłꍇlAexpire_full͗}optionk^OlpӂB
flag_lock = false, 'id', 'all'FAkL[l̂ݎcASc

ƁAȏ̂悤ɐ̂܂
LbVkL[l̂ݓǂő͓ǂ܂Ȃ́Aꕔset_infoōs
ikL[lieXiR[hj'id'Wi\jƂ邪A
  CM[L肩Ȃj

̃tOɂ͋@\Iɏd邪A܂悵ƂB
=end
